{{
  *********************************************************
  *  Font_convert - 1 bit to 2 bit font converter - v.1.0 * 
  *             (C) 2007 Gennady Palitsky                 *
  *          Jefferson Audio Video Systems                *
  *********************************************************
  v1.0 - 10 July 2007 - original version

  Supposed to buld alternating row buffers with 4-color characters from a built-in 2-color font
  Only one buffer is implemented, and only for the first field in interlaced video.

  The rest should be quite easy.
  For general operation see documentation for 'Font_border_generator_demo.spin'.
}}

VAR

  long  cog


PUB start(cptr)

'' Start convert - starts a cog
'' returns false if no cog available
''
''   c = pointer to buffers (along with other parameters

  stop
  cog := cognew(@entry, cptr) + 1


PUB stop

  if cog
    cogstop(cog~ - 1)


DAT

'*******************************
'* Assembly language routines  *
'*******************************

                        org     
'
' Entry
'
entry                   mov     t1,par           'save parameter
                        
                        rdlong  screen_addr,    t1      'read conv_screen - address of a screen buffer
                        add     t1,             #4      'next long
                        rdlong  rowbuf_addr,    t1      'read row_pointer - address of a row buffer
                        
                 'pins to test timing: P[0] - set when conversion starts, cleared when over
                 '                     P[1] may be used to test each character conversion time
                 'set pins 0,1 to out low
                        andn    outa,           #$03
                        or      dira,           #$03


wait_start       'wait for high P[2] - will be set by the tv-driver when display of previously developed row buffer starts
                 '                     and cleared somewhere along the way 
                        test    test_mask,      ina     wc     'check if P[2] is high
                  if_nc jmp     #wait_start                    'not high - wait more
                        or      outa,           #$01           '$$$ P[0] - set a test pin                 
read_row_data
                        mov     cur_screen_addr,        screen_addr             'restore starting screen address $$$
                        mov     cur_rowbuf_addr,        rowbuf_addr             'restore starting row buffer address $$$
                 'now we have screen address and row_buffer address - read data - all row data from a screen buffer
                 'NOTE: screen buffer is in WORDS, so we read 2 addresses at a time 15 times for a 30 characters row
'=========================                 
                        mov     buffer_counter, #15     '15 double characters
Doublecol_loop
                 
                        rdlong  col0,        cur_screen_addr                 'first 2 values in a screen buffer   
                        add     cur_screen_addr,        #4                   'prepare for next longcol
                        mov     col1,        col0                            '0,1
                        shr     col1,        #16                             'separate long (2 words into 2 longs
                                                                             'for character 0 and 1                                                                                                                                                                           
                        and     col0,        upper_word_mask
                        
                 '=== develop col0
                        test    col0,      odd_test_mask wc    'check bit 10 - c=1 for odd character in interleaved bitmap
                 'get a full address address of a first line of a bitmap
                        and     col0,           low10_mask     'remove color bits - fixed color is used, so we dont need it                 
                        shl     col0,           #6             'form an address to read from a built-in font (base - $8000) 
                        mov     line_counter,   #16            '16 lines of a bitmap to read                 
                        and     line_topL,      #0             'since there lines above for comparison, make empty ones 
                        and     line_topL3,     #0
                        and     line_topR,      #0
                 ' read current line - it's an odd field
:Line_loop0             rdlong  line_cur,       col0
                 if_c   shr     line_cur,       #1             'if bit 10 in a screen buffer data was set - even charater
                                                               'in interleaved bitmap - shift to right                 
                        add     col0,           #4             'to read next line - bottom - for comparison
                        rdlong  line_bot,       col0           'read bottom line                                 
                 if_c   shr     line_bot,       #1             'same as for a current line - shift if needed
                        add     col0,           #4             'ready to read next line
                 'clear interleaved  - clear bits where interleaved character resides in current and bottom lines
                        andn    line_cur,       high_mask
                        andn    line_bot,       high_mask
                 'now we effectively have line for 4-color character, 00 - was 0 in a 2-color
                 '                                                    01 - was 1 in a 2 color          
                 'arrrange shifts -

    'for simplicity I designate a pixel to compare the current pixel with by a clock terms              
    'to compare pixels in a current line with 10.30 o'clock neighbour pixels, we need to shift top line 1 bit to the right
    'to compare pixels in a current line with 12 o'clock neighbour pixels, we need to shift top line 1 bit to the left
    'to compare pixels in a current line with 1.30 o'clock neighbour pixels, we need to shift top line 3 bit to the left
                 
    'to compare pixels in a current line with 9 o'clock neighbour pixels, we need to shift current line 1 bit to the right
    'to compare pixels in a current line with 3 o'clock neighbour pixels, we need to shift current line 3 bit to the left

    'to compare pixels in a current line with 7.30 o'clock neighbour pixels, we need to shift bottom line 1 bit to the right
    'to compare pixels in a current line with 6 o'clock neighbour pixels, we need to shift bottom line 1 bit to the left
    'to compare pixels in a current line with 4.30 o'clock neighbour pixels, we need to shift bottom line 3 bit to the left

    'Note - though we only use odd lines, we need all lines to compare
                  
                        mov     line_curL,      line_cur                        'for current
                        shl     line_curL,      #1
                        mov     line_curL3,     line_cur
                        shl     line_curL3,     #3
                        mov     line_curR,      line_cur
                        shr     line_curR,      #1
                        mov     line_botL,      line_bot                        'for bottom
                        shl     line_botL,      #1
                        mov     line_botL3,     line_bot
                        shl     line_botL3,     #3
                        mov     line_botR,      line_bot
                        shr     line_botR,      #1

                 'get and write the final value to row_buffer
    'and here is the main idea - current pixel has a value 00 (background) or 01 (character)
    'pixels to compare with now have values have value 00 (background) or 10 (character)
    'ORing the current pixel with all the neighbours will give values:
    '      00 (backgroung) if neigher current pixel or neghbours are active character
    '      01 and 11 (character) if current pixel is an active character pixel, doesn't matter who the neighbours are
    '      10 (BORDER) if the current pixel is inactive one, but ANY of the neighbours is active 
                
                        or      line_cur,       line_curR                       'left neighbour
                        or      line_cur,       line_curL3                      'right neigbourt
                        or      line_cur,       line_topR                       'top left                        
                        or      line_cur,       line_topL                       'top                        
                        or      line_cur,       line_topL3                      'top right                        
                        or      line_cur,       line_botR                       'bottom left                        
                        or      line_cur,       line_botL                       'bottom                        
                        or      line_cur,       line_botL3                      'bottom right                        
                        
                        wrlong  line_cur,       cur_rowbuf_addr                 'write line to a current row buffer addr
                        add     cur_rowbuf_addr,    #4                          'prepare next address
                        mov     line_topL,      line_botL                       'we already have top shifted lines - they were bottooms 
                        mov     line_topL3,     line_botL3
                        mov     line_topR,      line_botR
                 'check line iteration counter
                        djnz    line_counter,   #:line_loop0
                        '========================================
                 '=== develop col1 - second WORD in a LONG screen buffer - you know the drill
                        test    col1,      odd_test_mask wc 'check bit 10 - c=1 for odd character
                        'get a full address address of a first line of a bitmap
                        and     col1,           low10_mask
                        shl     col1,           #6
                        'and     col1,           char_mask
                        'or      col1,           fontmap_mask
                        mov     line_counter,   #16
                        and     line_topL,      #0       'we don't need it for the last line - just for simplicity            
                        and     line_topL3,     #0
                        and     line_topR,      #0
                 ' read current line - it's an odd field
:Line_loop1             rdlong  line_cur,       col1
                 if_c   shr     line_cur,       #1
                        add     col1,           #4      'to read next line - bottom
                        rdlong  line_bot,       col1                                            
                 if_c   shr     line_bot,       #1
                        add     col1,           #4      'ready to read next line
                 'clear interleaved
                        andn    line_cur,       high_mask
                        andn    line_bot,       high_mask
                 'arrrange shifts
                        mov     line_curL,      line_cur                        'for current
                        shl     line_curL,      #1
                        mov     line_curL3,     line_cur
                        shl     line_curL3,     #3
                        mov     line_curR,      line_cur
                        shr     line_curR,      #1
                        mov     line_botL,      line_bot                        'for bottom
                        shl     line_botL,      #1
                        mov     line_botL3,     line_bot
                        shl     line_botL3,     #3
                        mov     line_botR,      line_bot
                        shr     line_botR,      #1

                 'get and write the final value to row_buffer
                        or      line_cur,       line_curR                       'left neighbour
                        or      line_cur,       line_curL3                      'right neigbourt
                        or      line_cur,       line_topR                       'top left                        
                        or      line_cur,       line_topL                       'top                        
                        or      line_cur,       line_topL3                      'top right                        
                        or      line_cur,       line_botR                       'bottom left                        
                        or      line_cur,       line_botL                       'bottom                        
                        or      line_cur,       line_botL3                      'bottom right                        
                        
                        wrlong  line_cur,       cur_rowbuf_addr                 'write line to a current row buffer addr
                        add     cur_rowbuf_addr,    #4                          'prepare next address
                        mov     line_topL,      line_botL
                        mov     line_topL3,     line_botL3
                        mov     line_topR,      line_botR
                 'check line iteration counter
                        djnz    line_counter,   #:line_loop1
                 'check buffer_counter counter
                        djnz    buffer_counter, #Doublecol_loop

                        andn    outa,           #$01                            'clear scope test pin

                        'wait for next coversion
                        jmp     #wait_start



' Initialized data
        test_mask       long    $00_00_00_04    'P[2] mask
        high_mask       long    %10101010_10101010_10101010_10101010
        odd_test_mask   long    %00000000_00000000_00000100_00000000
        high_clear_mask long    %00000011_11111111_11111111_11111111
        low10_mask      long    %00000000_00000000_00000011_11111111
        upper_word_mask long    $00_00_FF_FF
        char_mask       long    $FF
        fontmap_mask    long    $8000
        fonttest_mask   long    $10000
'        incr            long    $200
        
' Uninitialized data

saved_par               res     1
t1                      res     1
screen_addr             res     1
cur_screen_addr         res     1
temp                    res     1
char_addr               res     1
rowbuf_addr             res     1
cur_rowbuf_addr         res     1
line_counter            res     1
buffer_counter          res     1
line_top                res     1
line_topL               res     1
line_topL3              res     1
line_topR               res     1
line_cur                res     1
line_curL               res     1
line_curL3              res     1
line_curR               res     1
line_bot                res     1
line_botL               res     1
line_botL3              res     1
line_botR               res     1
longcol                 res     1
col0                 res     1
col1                 res     1

{
col2                 res     1
col3                 res     1
col4                 res     1
col5                 res     1
col6                 res     1
col7                 res     1
col8                 res     1
col9                 res     1
col10                res     1
col11                res     1
col12                res     1
col13                res     1
col14                res     1
col15                res     1
col16                res     1
col17                res     1
col18                res     1
col19                res     1
col20                res     1
col21                res     1
col22                res     1
col23                res     1
col24                res     1
col25                res     1
col26                res     1
col27                res     1
col28                res     1
col29                res     1
}







